/****	Boot Program Driver (BPD) for the MPA 16 PLUS Family of Radios
*****	Written in 1991 by Tim Kemp
*****	For:
*****		Ericsson/GE Mobile Communications
*****		Mountain View Road
*****		Lynchburg, VA
*****
*****	Command line to create this program:
*****
*****      cl flashmpa.c leafbpd /link c:\c50\comlib\ gfcs
*****
*****	Used Microsoft:
*****		C 6.00A
*****		LINK 5.10
*****		MASM 5.10A
*****
****/

/* CHANGE LIST

  Version:
  --------
    0) Changed serial communications libraries to Greenleaf com libraries.
    	This provided a more stable async platform and easier operation of
    	the port's control lines.
    
    1) Added routines to get and verify the radio personality.  This was a
    	part of a general upgrade of the mpa's flash arrangment to make it
    	a more solid operation.

    2) Added a routine to change the radio baud rate from the command line.
	This is done by looking for a parameter begining with a "/" and 
	expecting the baud rate to immeditately follow the "/" charactor.
	Also updated help message and changed some prompts to be more
	specific.

    3) Conventional radios lock up when fetching personality.  The problem was
    	that this program expects exactly 2000h bytes while the radio only
    	sends 1f20h bytes.  Things worked better when the byte count was
    	reduced to the 1f20 number.
    
    	Efforts were also made to improve the way the radio goes into program
    	mode.  There seem to be at least three radio states, each of which
    	needs to be handled differently.
*/

#include <stdio.h>
#include <asiports.h>

FILE *in;


/****	system_baud is the RS-232 baud rate to use
*****	combase is the comm port number
****/
extern int combase;
extern long system_baud;


/****	read one ASCII hex digit from the input file. Convert to binary.
****/
unsigned char getnibble()
{
	unsigned char c;
	do
	{
		if (feof(in))
		{
			printf("ERROR : File format.\n");
			exit1();
		}
		fread(&c,1,1,in);
	} while(c<'0' || (c>'9' && c<'A') || c>'F');
	if (c>'9') return(c-'A'+10);
	return(c-'0');
}


/****	read two hex digits by calling getnibble() twice
****/
unsigned char getbyte()
{
	unsigned char b;
	b = getnibble();
	return((b<<4) +getnibble());
}


/****	wait for a response byte from the radio. If a key is pressed,
*****	the program is cancelled.
****/
unsigned char inputbyte()
{
	unsigned char r;
	while(!chkkey())
	{
		if(!readcom(&r,1)) return(r);
	}
	
	while(chkkey()) getkey();
	printf("Programming cancelled.\n");
	printf("Beware, the personality was not restored\n");
	exit1();
}


/****	wait for a response from the radio, and interpret the response.
*****	1 = O.K. (return)
*****	0 = error (terminate program)
*****	anything else = bad response (terminate program)
****/
void getresp(error)
char *error;
{
	unsigned char r;
	r = inputbyte();
	if (r == 1) return;
	if (r == 0)
	{
		printf("%s\nProgramming terminated.\n",error);
		printf("Beware, the personality was not restored\n");
		exit1();
	}
	printf("Unexpected response from 80C51\nProgramming Terminated.\n");
	printf("Beware, the personality was not restored\n");
	exit1();
};


/****	calculate checksum for a binary version of an Intel Hex line
****/
unsigned char checksum(s)
unsigned char *s;
{
	unsigned char c;
	int i;

	c = 0;
	for (i=0; i<(s[0]+4); i++) c -= s[i];
	return(c);
}


/****	Parse a string of /bbbb for the baud rate.  Paramter p is a
 ****	pointer to that string.
 ****/
void parse_baud(p)
char *p;
{
    int i;

  /* Assume a 4 charactor number.  Shift over five positions to cover
     up the leading "/".  Need to shift 5 times to move the trailing "0"
     charactor over to properly end the new string.		*/
    for (i=0; i<5; i++)
    {
	*(p+i) = *(p+i+1);
    }
    system_baud = atol(p);
    printf("Radio baud rate changed to %i baud.\n", system_baud);

} /* End of newbaud */


/****	main program
****/
void main(argc,argv)
int argc;
char **argv;
{
	int i,j,k,retry,base,errors,status;
	int goteeprm;
	char *filename;
	unsigned char buffer[200],r,lr;

/****	Initialize:
*****	baud rate to 9600, default COMM port to 1 and set an illegal file name
****/
	system_baud = 9600;
	combase = 1;
	filename = "";

/****	this call is required by the library (TIM.LIB)
****/
	getmode();


/****	print title and parse command line
****/
	printf("\nGeneral Purpose 80C52 Flash Programmer Driver v1.3\n");
	printf("By Tim Kemp, Dave Dumonsau\n");
	printf("\n");
	printf("       Press any key to abort flashing !!\n\n");

	for (i=1; i<argc; i++)
	{
		if (argv[i][0] == '-')
		{
			combase = argv[i][1] - '0';
		}

		else if (argv[i][0] == '/')
		{
			parse_baud(&argv[i][0]);
		}
		else
		{
			filename = argv[i];
		}
	}

/****	if no file name, print help message. Exit.
****/
	if (!*filename)
	{
		printf(
			"usage : flashmpa file [-X] [/bbbb]\n"
			"where : 'file' is an Intel hex format file.\n"
			"        'X' is the COMM port number to use. (default = 1)\n"
			"        'bbbb' is a 4 digit baud rate. (default = 9600)\n"
			"note  : This program uses rts and dtr to switch the vpp supply\n"
		);
		exit(1);
	}

/****	if bad COMM port number, print error message. Exit.
****/
	if ((combase<1) ||(combase>4))
	{
		printf("\nERROR : %d is not a valid COMM port number\n",combase);
		exit(1);
	}

/****	open input file. If unsuccessful, print error message. Exit.
****/
	if (!(in = fopen(filename,"r")))
	{
		printf("ERROR : Can't open %s.\n",filename);
		exit(2);
	}


/****	try to open COMM port.  If error, print error message. Exit.
****/

    combase--;			/* Cause greeleaf starts countin at 0 */

    printf("The com ports are running out of the Greenleaf libraries\n");
    printf("Opening port COM%i...\n", combase+1);
    status = asiopen(combase, ASINOUT|BINARY|NORMALRX, 0x2000, 256, system_baud,
    			P_NONE, 1, 8, ON,  ON);
		/*                     |    |____ rts
				       |_________ dtr
							*/
    if (status < ASSUCCESS)
    {   printf("Can not open com port.   Exiting...\n");
	exit(1);
    }

    /* Let possible vpp die off */
    timer(2*18);  

/****	 Save the radio's personality
 ****/
	    goteeprm = FALSE;

	    for (i=0; i<2; i++)
            {   k = geteeprm(0);
		if ( (k == FALSE) || (k == OK) )
		    break;
	    }


	    if (k != FALSE)
	    {   for (i=0; i<5; i++)
	        {   goteeprm = FALSE;
	            if ( cmpeeprm(1) == TRUE )
	            {   goteeprm = TRUE;
		        break;
	            }
	            geteeprm(1);
	        }
	        if ( goteeprm == FALSE )
	            printf("Could not download personality\n");
	        else
                    printf("Current personality information has been saved\n");
	    }
	    else
	    {   goteeprm = FALSE;
	        printf("\nCould not download personality\n");
	    }


/****    Apply 12v to the radio
 ****/
    printf("Switching power with rts and dtr\n");
    asrts(combase, OFF);
    asdtr(combase, OFF);
    asiclear(combase, ASINOUT);

    /* Wait atleast 1 watchdog interval.  Try to give the impression
       that you are doing something  			*/
    printf("Waiting out a couple of watchdog intervals.");
    for (i=0; i<2*6; i++)
    {
	timer(9);
	putchar('.');
    }
    putchar('\r');
    for (i=0; i<78; i++)
	putchar(' ');
    putchar('\r');


/****	send radio password and look for response
****/
	for (i=0; i<250; i++)
	{
            asiclear(combase, ASINOUT);
	    r = 0;
	    docom("DEREK",&r,1);
	    if (r!=1)
	    {
		printf("\rRadio not responding.  Please cycle power on the radio.");
		if (chkkey())
		{   getkey();
		    printf("\nAbandoning trying to find radio\n");
		    printf("Beware, the personality was not restored\n");
		    exit1();
		}
	    }
	    else
		{ timer(2);
                  asiclear(combase, ASINOUT);
		  break;
		}
	}

	if (r!=1)
	{
	    printf("\nAbandoning trying to find radio\n");
	    printf("Beware, the personality was not restored\n");
	    exit1();
	}

/****	Clear a possible unable to find radio message.
*****/
	for (i=0; i<80; i++)
	    putchar(' ');
	putchar('\r');

/****	look for start of hex data (return followed by colon). If not found,
*****	print error message. Exit.
****/
	lr = '\n';
	fread(&r,1,1,in);
	while((r != ':') || (lr != '\n'))
	{
		if (feof(in))
		{
			printf("ERROR : No data in %s\n",filename);
			printf("Beware, the personality was not restored\n");
			exit1();
		}
		lr = r;
		fread(&r,1,1,in);
	}

/****	Zero part
****/
	printf("Zeroing flash part...\n");
	r = 2;
	outncom(&r,1);
	getresp("Unable to zero flash part.");

/****	Erase part
****/
	printf("Erasing flash part...\n");
	r = 1;
	outncom(&r,1);
	getresp("Unable to erase flash part.");

/****	Program part
****/
	printf("Programming part...\nAddress :\n");

/****	read number of bytes on line
****/
	buffer[0] = getbyte();

/****	each time through loop handles one line. continues until no more data
****/
	while(buffer[0])
	{
	
/****	read in one line
****/
		for(i=1; i<buffer[0]+5; i++) buffer[i] = getbyte();

/****	calculate checksum and compare to checksum in file. If error,
*****	print error message. Exit.
****/
		if (buffer[buffer[0]+4] != (r = checksum(buffer)))
		{
			printf("\n\n\bERROR : File contains bad checksum\n");
			printf("Beware, the personality was not restored\n");
			exit1();
		}

/****	send one line to radio to burn into Flash EPROM.  Retry transmission
*****	up to 5 times.  If no good, print error message and exit.
****/
		r = 2;
		i = 0;
		retry = 0;
		while ((r == 2) && (retry < 5))
		{
			r = 0;
			outncom(&r,1);
			outncom(buffer,buffer[0]+5);
			r = inputbyte();
			retry++;
		}
		
		if (r == 0)
		{
			printf("\nError while programming flash part.\nTerminating programming.\n");
			printf("Beware, the personality was not restored\n");
			exit1();
		}

/****	print last line's address as status indicator
****/
		printf("\t%04.4X\r",(long)buffer[1]*256L+(long)buffer[2]);

/****	read number of bytes on line
****/
		buffer[0] = getbyte();
	}

/****	programming finished. Print message and turn off vpp.
****/
	printf("\n\nRadio programmed.\n");
	printf("Turning off power with rts and dtr\n");
	asrts(combase, ON);
	asdtr(combase, ON);
	if ( !goteeprm )
	{   printf("Beware, the personality was not restored\n");
	    exit1();
	}
	timer(5 * 18);

    for (i=0; i<2; i++)
    {   if (cmpeeprm(0) == TRUE)
        {   printf("The eeprom was not corrupted\n");
	    exit1();
        }
    }


/***** The eeprom was corrupted.  Reprogram it.
 ****/
    printf("The eeprom was corrupted\n");


    for (i=0; i<10; i++)
    {
        puteeprm();
	timer(2);
        if (cmpeeprm(1) == TRUE)
        {
	    printf("The eeprom has been restored\n");
	    exit1();
        }

        printf("The eeprom write has failed.\n");
        printf("The eeprom is still corrupted\n");
    }

    exit1();

}   /* End of main() */


/****	exit from the program: close all files, give the interrupts back to
*****	the previous owner
****/
exit1()
{
	asrts(combase, ON);
	asdtr(combase, ON);

	asiputc(combase, 'G');
	asiputc(combase, '9');
	timer(2);

	fcloseall();
	doexit("");
}
